#!/usr/bin/python
#
# The kernel can be protected in several ways:
#
# 1. Normal kernels used during development, on devices that don't
#    need to be secure and by customers who wish to take advantage of
#    their GPL rights are simply protected by a SHA256 hash in an ELF
#    section of type 0x60425301.
#
# 2. Officially-released kernels that need to be secure are also
#    protected by at most one "legacy" 512-bit signature (in section
#    0x60425300) and one or more full 2048-bit signatures (in section
#    0x60425302.)

import sys, os, mmap, struct, hashlib, binascii, zlib, subprocess, shlex

asnoid_sha256 = "3031300d060960864801650304020105000420"

if len(sys.argv) < 3:
    print "usage: %s --sha256 <filename>" % sys.argv[0]
    print "usage: %s --sha256-rsa <key> <filename>" % sys.argv[0]
    print "       %s --adler32 <filename>" % sys.argv[0]
    sys.exit(1)

class Sha256Hash:
    def __init__(self):
        self.sha = hashlib.sha256()
    def update(self, d):
        self.sha.update(d)
    def digest(self):
        return self.sha.digest()

class Adler32Hash:
    def __init__(self):
        self.adler = zlib.adler32("")
    def update(self, d):
        self.adler = zlib.adler32(d, self.adler) & 0xffffffff
    def digest(self):
        return binascii.unhexlify("%08x" % (self.adler & 0xffffffff)) + '\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0';

def sign_hash(d, k):
    cmd = "openssl rsautl -sign -pkcs -inkey " + k
    p = subprocess.Popen(shlex.split(cmd), stdin=subprocess.PIPE, stdout=subprocess.PIPE)

    # Apply PKCS#1 standard padding for SHA256 just in case we ever
    # want to check these hashes using libgcrypt. The remaining
    # padding will be applied by rsautl.
    (output, err) = p.communicate(asnoid_sha256.decode("hex") + d)
    if p.returncode != 0:
        raise Exception, "signing failed"
    return output

sign = 0

if sys.argv[1] == "--adler32":
    # Not used anywhere
    sha = Adler32Hash()
    alg = 1
    fn = sys.argv[2]
elif sys.argv[1] == "--sha256":
    # Used on cheetah for insecure kernels
    sha = Sha256Hash()
    alg = 0
    fn = sys.argv[2]
    section_type = 0x60425301
elif sys.argv[1] == "--sha256-legacy":
    # Used on panther because it expects the algorithm to be encoded in the bottom nibble
    sha = Sha256Hash()
    alg = 0
    fn = sys.argv[2]
    section_type = 0x60425300
elif sys.argv[1] == "--sha256-rsa":
    # Used on production cheetah units
    sha = Sha256Hash()
    sign = 2
    alg = 0
    keyfile = sys.argv[2]
    fn = sys.argv[3]
    section_type = 0x60425302
elif sys.argv[1] == "--sha256-rsa-legacy":
    # Used on early cheetah units
    sha = Sha256Hash()
    sign = 1
    alg = 0
    keyfile = sys.argv[2]
    fn = sys.argv[3]
    section_type = 0x60425300
else:
    print "Unknown hash type"
    sys.exit(1)

written = False

fp = os.open(fn, os.O_RDWR)
m = mmap.mmap(fp, 0)

(ident, type, machine, version, entry, phoff, shoff, flags, ehsize, phentsize, phnum, shentsize) = struct.unpack("16sHHIIIIIHHHH", m[0:48])
phend = phoff + (phentsize * phnum)

print "phoff = 0x%x" % phoff
print "phend = 0x%x" % phend

sha.update(m[0:52])
sha.update(m[phoff:phend])

def sign_section(section_type):
    PT_LOAD = 1
    PT_BRIGHTSIGN_DEVICETREE_GROUP = 0x60425400
    PT_BRIGHTSIGN_DEVICETREE_MASK = 0xffffff00

    if (section_type == PT_LOAD):
        return True
    return (section_type & PT_BRIGHTSIGN_DEVICETREE_MASK == PT_BRIGHTSIGN_DEVICETREE_GROUP)

while (phoff + phentsize <= phend):
    (p_type, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_flags, p_align) = struct.unpack("IIIIIIII", m[phoff:phoff+phentsize])
    if sign_section(p_type):
        print "Mixing in section of type " + hex(p_type) + ", length " + hex(p_filesz)
        sha.update(m[p_offset:p_offset+p_filesz])
    elif (p_type == section_type):
        if ord(m[p_offset+3:p_offset+4]) == 0 and not written:
            hash = sha.digest()
            print "Calculated hash: " + binascii.hexlify(hash)
            if sign:
                hash = sign_hash(hash, keyfile)
                print "Signed data: " + binascii.hexlify(hash)
            if (p_filesz < 4+len(hash)):
                print "hash section too small for data!"
                sys.exit(1)
            m[p_offset:p_offset+1] = chr(sign)
            m[p_offset+3:p_offset+4] = chr(1)
            m[p_offset+4:p_offset+(4+len(hash))] = hash
            written = True
    else:
        print "Skipping section of type " + hex(p_type)
    phoff += phentsize

if not written:
    print "Unable to write hash"
    sys.exit(2)
